package ru.myitschool.work.ui.profile

import android.content.DialogInterface
import android.content.res.ColorStateList
import android.os.Bundle
import android.view.View
import android.widget.TextView
import androidx.annotation.Nullable
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import com.squareup.picasso.Picasso
import dagger.hilt.android.AndroidEntryPoint
import ru.myitschool.work.R
import ru.myitschool.work.databinding.FragmentProfileBinding
import ru.myitschool.work.ui.entrylist.EntryListDestination
import ru.myitschool.work.ui.login.LoginDestination
import ru.myitschool.work.ui.qr.result.QrResultDestination
import ru.myitschool.work.ui.qr.scan.QrScanDestination
import ru.myitschool.work.ui.searchuser.SearchUserDestination
import ru.myitschool.work.utils.collectWhenStarted
import ru.myitschool.work.utils.visibleOrGone


@AndroidEntryPoint
class ProfileFragment : Fragment(R.layout.fragment_profile) {
    private var _binding: FragmentProfileBinding? = null
    private val binding: FragmentProfileBinding get() = _binding!!

    private val viewModel: ProfileViewModel by viewModels()

    private lateinit var swipeRefreshLayout: SwipeRefreshLayout

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentProfileBinding.bind(view)

        initCallback()
        subscribe()

        swipeRefreshLayout = binding.swipeRefreshLayout

        swipeRefreshLayout.setOnRefreshListener {
            viewModel.updateUserInfo()
        }
    }


    private fun subscribe() {
        viewModel.state.collectWhenStarted(this) { state ->
            binding.showState.visibleOrGone(state is ProfileViewModel.State.Show)
            binding.error.visibleOrGone(state is ProfileViewModel.State.Error)
            binding.loading.visibleOrGone(state is ProfileViewModel.State.Loading)

            when(state) {
                is ProfileViewModel.State.Loading -> {
                    swipeRefreshLayout.isRefreshing = true
                }
                is ProfileViewModel.State.Error -> {
                    swipeRefreshLayout.isRefreshing = false
                    binding.error.text = state.errorText
                }
                is ProfileViewModel.State.Show -> {
                    swipeRefreshLayout.isRefreshing = false
                    binding.position.text = state.position
                    binding.lastEntry.text = state.lastEntry
                    binding.fullname.text = state.fullname
                    if (state.admin){
                        binding.admin.visibility = View.VISIBLE
                    } else {
                        binding.admin.visibility = View.GONE
                    }
                    if (state.blocked) {
                        binding.scan.isEnabled = false
                        binding.scan.imageTintList = ColorStateList.valueOf(ContextCompat.getColor(requireContext(), R.color.white))
                        binding.scan.backgroundTintList = ColorStateList.valueOf(ContextCompat.getColor(requireContext(), R.color.ErrorRed))
                    }
                    Picasso.get()
                        .load(state.imageUrl)
                        .error(R.drawable.ic_no_img)
                        .into(binding.photo)
                }
            }
        }

        viewModel.action.collectWhenStarted(this) { action ->
            when(action) {
                is ProfileViewModel.Action.Logout -> {
                    findNavController().navigate(LoginDestination) {
                        popUpTo<ProfileDestination> { inclusive = true }
                    }
                }
                is ProfileViewModel.Action.OpenLogin -> {

                    val builder = AlertDialog.Builder(requireContext())

                    builder.setTitle(ContextCompat.getContextForLanguage(requireContext()).getString(R.string.profile_logout_dialog));
                    builder.setMessage(ContextCompat.getContextForLanguage(requireContext()).getString(R.string.profile_logout_text_dialog))
                    builder.setPositiveButton(
                        ContextCompat.getContextForLanguage(requireContext()).getString(R.string.logout),
                        DialogInterface.OnClickListener { dialogInterface, i ->
                            viewModel.logout()
                             dialogInterface.dismiss()
                        })
                    builder.setNegativeButton(
                        ContextCompat.getContextForLanguage(requireContext()).getString(R.string.cancel),
                        DialogInterface.OnClickListener { dialogInterface, i ->
                            dialogInterface.dismiss()

                        })

                    val dialog = builder.create()



                    dialog.setOnShowListener {
                        val titleView = dialog.findViewById<TextView>(android.R.id.title)
                        val messageView = dialog.findViewById<TextView>(android.R.id.message)



                        titleView?.setTextAppearance(R.style.Theme_UiTemplate_TextH2)
                        messageView?.setTextAppearance(R.style.Theme_UiTemplate_TextH4)



                        val positiveButton = dialog.getButton(AlertDialog.BUTTON_POSITIVE)
                        val negativeButton = dialog.getButton(AlertDialog.BUTTON_NEGATIVE)


                        positiveButton.setTextColor(ContextCompat.getColor(requireContext(), R.color.ErrorRed) )
                        negativeButton.setTextColor(ContextCompat.getColor(requireContext(), R.color.AccentBlue) )
                    }

                    // Установка кнопок


                    // Показать диалог
                    dialog.show()
         }

                is ProfileViewModel.Action.OpenLog -> {
                    findNavController().navigate(EntryListDestination) {
                        popUpTo<EntryListDestination> { inclusive = true }
                    }
                }

                is ProfileViewModel.Action.OpenScan -> {
                    findNavController().apply {
                        navigate(QrResultDestination)
                        navigate(QrScanDestination)
                    }
                }

                is ProfileViewModel.Action.OpenSearch -> {
                    findNavController().navigate(SearchUserDestination) {
                        popUpTo<SearchUserDestination> { inclusive = true }
                    }
                }

            }
        }
    }

    private fun initCallback() {
        binding.logout.setOnClickListener { viewModel.clickLogout() }
        binding.scan.setOnClickListener { viewModel.clickScan() }
        binding.entryList.setOnClickListener{ viewModel.clickLog()}
        binding.admin.setOnClickListener { viewModel.clickSearch() }
    }
    //Вызывается перед уничтожением фрагмента
    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        // Сохранение состояния
        viewModel.updateUserInfo()
    }
    //Когда востанавлиеваетсья
    override fun onViewStateRestored(@Nullable savedInstanceState: Bundle?) {
        super.onViewStateRestored(savedInstanceState)
        // Восстановление состояния
        viewModel.updateUserInfo()
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }
}